
#property library
#include <stderror.mqh>


/**
 * Assign an integer array to a position of a multi-dimensional integer array (in short: dest_array[i] = values[] for arrays of arrays).
 *
 * @param  int array[][] - multi-dimensional array to modify
 * @param  int i         - position to modify
 * @param  int values[]  - array to assign to the specified position (sizes/dimensions must match)
 *
 * @return int - error status
 */
int ArraySetIntArray(int array[][], int i, int values[]) {
   if (ArrayDimension(array) != 2)  return(catch("ArraySetIntArray(1)   illegal dimensions of parameter array = "+ ArrayDimension(array), ERR_INCOMPATIBLE_ARRAYS));
   if (ArrayDimension(values) != 1) return(catch("ArraySetIntArray(2)   too many dimensions of parameter values = "+ ArrayDimension(values), ERR_INCOMPATIBLE_ARRAYS));
   int dim1 = ArrayRange(array, 0);
   int dim2 = ArrayRange(array, 1);
   if (ArraySize(values) != dim2)   return(catch("ArraySetIntArray(3)   array size mis-match of parameters array and values: array["+ dim1 +"]["+ dim2 +"] / values["+ ArraySize(values) +"]", ERR_INCOMPATIBLE_ARRAYS));
   if (i < 0 || i >= dim1)          return(catch("ArraySetIntArray(4)   illegal parameter i = "+ i, ERR_INVALID_FUNCTION_PARAMVALUE));

   CopyMemory(GetBufferAddress(array) + i*dim2*4, GetBufferAddress(values), dim2*4);
   return(GetLastError());
}


// -----------------------------------------------------------------------------------------------------------------
// error handling, little helpers etc., unrelated to ArraySetIntArray()
// -----------------------------------------------------------------------------------------------------------------

/**
 * Copies a block of memory from one location to another. Opposite to the original Windows function the blocks specified
 * by source and destination address may overlap.
 *
 * @param  int destination - address of copy destination
 * @param  int source      - address of block to copy
 * @param  int bytes       - size in bytes of block to copy
 */
void CopyMemory(int destination, int source, int bytes) {
   RtlMoveMemory(destination, source, bytes);
}


#import "ExpertSample.ex4"
   int  GetBufferAddress(int buffer[]);

#import "kernel32.dll"
   void RtlMoveMemory(int destAddress, int srcAddress, int bytes);

#import "user32.dll"
   int  MessageBoxA(int hWnd, string lpText, string lpCaption, int style);
#import
#define MB_OK              0x00000000
#define MB_ICONERROR       0x00000010

#import "winmm.dll"
   bool PlaySoundA(string lpSound, int hMod, int fSound);
#import
#define SND_ASYNC                0x01     // play asynchronously
#define SND_FILENAME       0x00020000     // name is file name


/**
 * Error checking/handling
 *
 * @param  string location - location identifier of the error occured
 * @param  int    error    - enforce a specific error by hand (default: ERR_NO_ERROR)
 *
 * @return int - the error code itself
 */
int catch(string location, int error=ERR_NO_ERROR) {
   if (error == ERR_NO_ERROR)
      error = GetLastError();

   if (error != ERR_NO_ERROR) {
      string message = location +"  [error code: "+ error +"]";

      // log error
      Alert("ERROR:   ", Symbol(), ",  ", WindowExpertName(), "::", message);

      // show error
      if (IsTesting()) {
         // force a sound and a message box even if MT4 wouldn't do so (Alert()/MessageBox() are not available in Tester)
         PlaySoundA(TerminalPath() +"\\sounds\\alert.wav", NULL, SND_FILENAME|SND_ASYNC);
         MessageBoxA(NULL, "ERROR in "+ WindowExpertName() +"::"+ message, "Strategy Tester", MB_ICONERROR|MB_OK);
      }
   }
   return(error);
}
